# ------------------------------------------------------------------------------------------------ #
# MIT License                                                                                      #
#                                                                                                  #
# Copyright (c) 2020, Microsoft Corporation                                                        #
#                                                                                                  #
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software    #
# and associated documentation files (the "Software"), to deal in the Software without             #
# restriction, including without limitation the rights to use, copy, modify, merge, publish,       #
# distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the    #
# Software is furnished to do so, subject to the following conditions:                             #
#                                                                                                  #
# The above copyright notice and this permission notice shall be included in all copies or         #
# substantial portions of the Software.                                                            #
#                                                                                                  #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING    #
# BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND       #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,     #
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,   #
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.          #
# ------------------------------------------------------------------------------------------------ #

r"""
Value Losses
============

.. autosummary::
    :nosignatures:

    coax.value_losses.mse
    coax.value_losses.huber
    coax.value_losses.logloss
    coax.value_losses.logloss_sign
    coax.value_losses.quantile_huber

----

This is a collection of loss functions that may be used for learning a value function. They are just
ordinary loss functions known from supervised learning.


Object Reference
----------------

.. autofunction:: coax.value_losses.mse
.. autofunction:: coax.value_losses.huber
.. autofunction:: coax.value_losses.logloss
.. autofunction:: coax.value_losses.logloss_sign
.. autofunction:: coax.value_losses.quantile_huber

"""

from ._losses import mse, huber, logloss, logloss_sign, quantile_huber


__all__ = (
    'mse',
    'huber',
    'logloss',
    'logloss_sign',
    'quantile_huber'
)
